@@ -1,4 +1,4 @@
-/* @(#)util.h	1.7 11/05/06 2011 J. Schilling */
+/* @(#)util.h	1.8 11/11/07 2011 J. Schilling */
 /*
  *	Copyright (c) 1986 Larry Wall
  *	Copyright (c) 2011 J. Schilling
@@ -16,7 +16,9 @@ extern	void	fatal	__PR((const char *fmt, ...)) __printflike__(1, 2);
 extern	void	ask	__PR((const char *fmt, ...)) __printflike__(1, 2);
 extern	void	set_signals __PR((int reset));
 extern	void	ignore_signals __PR((void));
-extern	void	makedirs __PR((register char *filename, bool striplast));
+#ifndef	_SCHILY_SCHILY_H
+extern	int	makedirs __PR((register char *filename, mode_t mode, bool striplast));
+#endif
 extern	char	*fetchname __PR((char *at, int strip_leading,
 						int assume_exists,
 						bool *isnulldate));
