@@ -2,11 +2,13 @@
  * CDDL HEADER START
  *
  * The contents of this file are subject to the terms of the
- * Common Development and Distribution License (the "License").
- * You may not use this file except in compliance with the License.
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
  *
- * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -25,12 +27,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2014 J. Schilling
  *
- * @(#)admin.c	1.73 11/11/01 J. Schilling
+ * @(#)admin.c	1.83 14/08/13 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)admin.c 1.73 11/11/01 J. Schilling"
+#pragma ident "@(#)admin.c 1.83 14/08/13 J. Schilling"
 #endif
 /*
  * @(#)admin.c 1.39 06/12/12
@@ -97,13 +99,11 @@ should appear exactly as they do in the msgid string:
 */
 
 # define MAXNAMES 9
-# define COPY 0
-# define NOCOPY 1
 
 static char	stdin_file_buf [20];
 static char	*ifile, *tfile;
 static char	*dir_name;
-static time_t	ifile_mtime;
+static struct timespec	ifile_mtime;
 static int	Ncomma = 0;
 static int	Nsdot = 1;
 static int	Nsubd;
@@ -171,7 +171,9 @@ char *argv[];
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
 	
-	/* 
+	sccs_setinsbase(INS_BASE);
+
+	/*
 	 * Set directory to search for general l10n SCCS messages.
 	 */
 #ifdef	PROTOTYPES
@@ -195,6 +197,10 @@ char *argv[];
 	*/
 	set_clean_up(clean_up);
 	Fflags = FTLMSG | FTLCLN | FTLEXIT;
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 
 	testklt = 1;
 
@@ -230,7 +236,7 @@ char *argv[];
 				*  this takes care of options given after
 				*  file names.
 				*/
-			if((c == EOF)) {
+			if (c == EOF) {
 			   if (optind < argc) {
 				/* if it's due to -- then break; */
 			       if(argv[j][0] == '-' &&
@@ -259,7 +265,8 @@ char *argv[];
 				   if ((Statbuf.st_mode & S_IFMT) == S_IFDIR) {
 					direrror(ifile, c);
 				   } else {
-					ifile_mtime = Statbuf.st_mtime;
+					ifile_mtime.tv_sec = Statbuf.st_mtime;
+					ifile_mtime.tv_nsec = stat_mnsecs(&Statbuf);
 				   }
 				}
 				break;
@@ -546,6 +553,12 @@ char *argv[];
 			    (had[LOWER(c)? c-'a' : NLOWER+c-'A']++ && testklt++))
 				fatal(gettext("key letter twice (cm2)"));
 	}
+#ifdef	SCCS_V6_ENV
+	if (versflag != 6) {
+		if (getenv("SCCS_V6"))
+			versflag = 6;
+	}
+#endif
 
 	for(i=1; i<argc; i++){
 		if(argv[i]) {
@@ -683,7 +696,7 @@ char	*afile;
 	if (HADI)
 		HADN = 1;
 	if (HADI || HADN) {
-			if (VFLAG && had_flag[CMFFLAG - 'a'])
+		if (VFLAG && had_flag[CMFFLAG - 'a'])
 			fatal(gettext("Can't have two verification routines."));
 
 		if (HADM && !VFLAG && !had_flag[CMFFLAG - 'a'])
@@ -722,7 +735,7 @@ char	*afile;
 		   check for a corrupted file.
 		*/
 		if ((pid = vfork()) < 0)
-			fatal(gettext("cannot fork, try again"));
+			efatal(gettext("cannot fork, try again"));
 		if (pid == 0) {		/* child */
 			/*
 			   perform 'val' with appropriate keyletters
@@ -736,7 +749,7 @@ char	*afile;
 #ifdef	HAVE_VFORK
 			Fflags |= FTLVFORK;
 #endif
-			fatal(SccsError);
+			efatal(SccsError);
 		}
 		else {
 			wait(&status);	   /* wait on status from 'execlp' */
@@ -753,7 +766,7 @@ char	*afile;
 	uname(&un);
 	uuname = un.nodename;
 	if (!HADH && lockit(copy(auxf(afile,'z'),Zhold),SCCS_LOCK_ATTEMPTS,getpid(),uuname))
-		fatal(gettext("cannot create lock file (cm4)"));
+		efatal(gettext("cannot create lock file (cm4)"));
 
 	if (fexists) { /* modifying */
 		int	cklen = 8;
@@ -772,10 +785,6 @@ char	*afile;
 			gpkt.p_line[cklen]   = '\000';
 		}
 	}
-
-
-	
-
 	else {
 		if ((int) strlen(sname(afile)) > MAXNAMLEN) {
 			sprintf(SccsError, gettext("file name is greater than %d characters"),
@@ -810,23 +819,10 @@ char	*afile;
 		/*
 		 * Initialize global meta data
 		 */
-		if (versflag == 6 && setrhome != NULL &&
-		    afile[0] != '/' &&
-		    !(afile[0] == '.' && afile[1] == '.' && afile[2] == '/') &&
-		    !strstr(afile, "/../")) {
-			char	*p;
-
-			sprintf(line, "%s%s%s%s%s",
-				homedist > 0 ? cwdprefix:"",
-				homedist > 0 ? "/": "",
-				dir_name, *dir_name ? "/":"",
-				auxf(afile, 'g'));
-			p = fmalloc(size(line));
-			strcpy(p, line);
-			gpkt.p_init_path = p;
-		}
-		if (versflag == 6)
+		if (versflag == 6) {
+			set_init_path(&gpkt, afile, dir_name);
 			urandom(&gpkt.p_rand);
+		}
 	}
 
 	if (!HADH)
@@ -889,13 +885,15 @@ char	*afile;
 			 dt.d_sid.s_br = dt.d_sid.s_seq = 0;
 			}
 		dtime(&dt.d_dtime);		/* get time and date */
-                if (HADN && HADI && (HADO || HADQ) && (ifile_mtime != 0)) {
+                if (HADN && HADI && (HADO || HADQ) &&
+		    (ifile_mtime.tv_sec != 0)) {
                         /*
-			 * When specifying -o (oroginal date) and
+			 * When specifying -o (original date) and
 			 * for NSE when putting existing file under sccs the
                          * delta time is the mtime of the clear file.
                          */
-			time2dt(&dt.d_dtime, ifile_mtime, 0);
+			time2dt(&dt.d_dtime,
+				ifile_mtime.tv_sec, ifile_mtime.tv_nsec);
                 }
 
 		copy(logname(),dt.d_pgmr);	/* get user's name */
@@ -961,7 +959,7 @@ char	*afile;
 		For old file, copy to x-file until user-name section
 		is found.
 		*/
-		flushto(&gpkt,BUSERNAM,COPY);
+		flushto(&gpkt, BUSERNAM, FLUSH_COPY);
 
 	/*
 	Write user-names to be added to list of those
@@ -1005,14 +1003,14 @@ char	*afile;
 		*/
 		sprintf(line,CTLSTR,CTLCHAR,EUSERNAM);
 		putline(&gpkt,line);
-	}
-	else
+	} else {
 		/*
 		For old file, copy to x-file until end of
 		user-names section is found.
 		*/
 		if (!HADE)
-			flushto(&gpkt,EUSERNAM,COPY);
+			flushto(&gpkt, EUSERNAM, FLUSH_COPY);
+	}
 
 	/*
 	For old file, read flags and their values (if any), and
@@ -1097,7 +1095,7 @@ char	*afile;
 		if (had_flag[VALFLAG - 'a'] && !rm_flag[VALFLAG - 'a'])
 			fatal (gettext("Can't use -fz with -fv."));
 	}
-	for (k = 0; k < NFLAGS; k++)
+	for (k = 0; k < NFLAGS; k++) {
 		if (had_flag[k]) {
 			int	i;		/* for flag string cleanup */
 
@@ -1148,6 +1146,7 @@ char	*afile;
 				}
 			}
 		}
+	}
 
 	if (HADN) {
 		if (HADI || HADB) {
@@ -1163,6 +1162,10 @@ char	*afile;
 				CTLCHAR, FLAG, ENCODEFLAG, Encoded);
 			putline(&gpkt,line);
 		}
+		/*
+		 * Writing out SCCS v6 flags belongs here.
+		 */
+
 		putmeta(&gpkt);
 
 		/*
@@ -1170,14 +1173,38 @@ char	*afile;
 		*/
 		sprintf(line,CTLSTR,CTLCHAR,BUSERTXT);
 		putline(&gpkt,line);
-	}
-	else
+	} else {
 		/*
-		Write out BUSERTXT record which was read in
-		above loop that processes flags.
-		*/
+		 * Check where the above loop that processes flags stopped:
+		 */
 		gpkt.p_wrttn = 0;
-		putline(&gpkt,(char *) 0);
+		/*
+		 * Copy over SCCS v6 flags.
+		 */
+		while (gpkt.p_line_length > 1 &&
+			    gpkt.p_line[0] == CTLCHAR &&
+			    gpkt.p_line[1] == NAMEDFLAG) {
+			getline(&gpkt);
+		}
+
+		/*
+		 * Copy over SCCS v6 global metadata.
+		 */
+		while (gpkt.p_line_length > 1 &&
+			    gpkt.p_line[0] == CTLCHAR &&
+			    gpkt.p_line[1] == GLOBALEXTENS) {
+			getline(&gpkt);
+		}
+
+		/*
+		 * Write out everything until the BUSERTXT record.
+		 * This includes possible future extensions.
+		 */
+		if (gpkt.p_line[0] == CTLCHAR && gpkt.p_line[1] == BUSERTXT)
+			putline(&gpkt,(char *) 0);
+		else
+			flushto(&gpkt, BUSERTXT, FLUSH_COPY);
+	}
 
 	/*
 	Get user description, copy to x-file.
@@ -1192,6 +1219,12 @@ char	*afile;
 			(void)fgetchk(iptr, tfile, &gpkt, 0);
 			fclose(iptr);
 			iptr = NULL;
+			/*
+			 * fgetchk() did set p_ghash and in case that the
+			 * file has zero size or -n is used, p_ghash may never
+			 * be set up again. So we need to clear it here.
+			 */
+			gpkt.p_ghash = 0;
 		   }
 		}
 
@@ -1200,7 +1233,7 @@ char	*afile;
 		commentary. (i.e., don't copy it to x-file.)
 		*/
 		if (!HADN)
-			flushto(&gpkt,EUSERTXT,NOCOPY);
+			flushto(&gpkt, EUSERTXT, FLUSH_NOCOPY);
 	}
 
 	if (HADN) {		/*   N E W  F I L E   */
@@ -1931,7 +1964,8 @@ static char	Nhold[MAXPATHLEN];
 			if ((Statbuf.st_mode & S_IFMT) == S_IFDIR) {
 				direrror(afile, 'i');
 			} else {
-				ifile_mtime = Statbuf.st_mtime;
+				ifile_mtime.tv_sec = Statbuf.st_mtime;
+				ifile_mtime.tv_nsec = stat_mnsecs(&Statbuf);
 			}
 		} else {
 			xmsg(afile,NOGETTEXT("admin"));
@@ -1946,20 +1980,14 @@ static char	Nhold[MAXPATHLEN];
 				len = sizeof (Dir);
 			strlcpy(Dir, afile, len); /* replace last '/' by '\0' */
 			/*
-			 * fixpath() (formerly abspath() from AT&T) is buggy: it
-			 * is unable to remove "./" at the beginning and it
-			 * returns -1 leaving a corrupted path buffer in case
-			 * it contained too many "/../" entries.
+			 * We need a path free of symlink components.
 			 * resolvepath() is available as syscall on Solaris,
-			 * currently missing elsewhere. We however need a path
-			 * free of symlink components.
+			 * or as user space implementation in libschily.
 			 */
-#ifdef	HAVE_RESOLVEPATH
-			if (resolvepath(Dir, Dir, sizeof (Dir)) == -1)
-#else
-			if (fixpath(Dir) == (char *)-1)
-#endif
-				fatal(gettext("path conversion error (cm12)"));
+			if ((len = resolvepath(Dir, Dir, sizeof (Dir))) == -1)
+				efatal(gettext("path conversion error (cm12)"));
+			else if (len >= sizeof (Dir))
+				fatal(gettext("resolved path too long (cm11)"));
 		}
 		if (Dir[0] != '\0' && (dfd < 0 || chdir(Dir) < 0)) {
 			cat(Nhold, Dir, *Dir?"/":"", Nprefix, sn, (char *)0);
@@ -1990,18 +2018,20 @@ static char	Nhold[MAXPATHLEN];
 		if (np > afile) {
 			np[-1] = '\0';
 			if (dfd >= 0) {
+				int	len;
+
 				if (afile[0] == '\0')
 					copy("/", Dir);
 				else
 					copy(afile, Dir);
-#ifdef	HAVE_RESOLVEPATH
-				if (resolvepath(Dir, Dir, sizeof (Dir)) == -1)
-#else
-				if (fixpath(Dir) == (char *)-1)
-#endif
-					fatal(gettext("path conversion error (cm12)"));
+
+				if ((len = resolvepath(Dir, Dir, sizeof (Dir))) == -1)
+					efatal(gettext("path conversion error (cm12)"));
+				else if (len >= sizeof (Dir))
+					fatal(gettext("resolved path too long (cm11)"));
+
 				if (chdir(Dir) < 0)
-					fatal("Chdir");
+					efatal("Chdir");
 				dir_name = Dir;
 				if (dir_name[0] == '.' && dir_name[1] == '/')
 					dir_name += 2;
@@ -2020,7 +2050,8 @@ static char	Nhold[MAXPATHLEN];
 			if ((Statbuf.st_mode & S_IFMT) == S_IFDIR) {
 				direrror(ifile, 'i');
 			} else {
-				ifile_mtime = Statbuf.st_mtime;
+				ifile_mtime.tv_sec = Statbuf.st_mtime;
+				ifile_mtime.tv_nsec = stat_mnsecs(&Statbuf);
 			}
 		} else {
 			xmsg(afile,NOGETTEXT("admin"));
