@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * This file contains modifications Copyright 2006-2014 J. Schilling
  *
- * @(#)comb.c	1.16 11/09/11 J. Schilling
+ * @(#)comb.c	1.23 14/08/09 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)comb.c 1.16 11/09/11 J. Schilling"
+#pragma ident "@(#)comb.c 1.23 14/08/09 J. Schilling"
 #endif
 /*
  * @(#)comb.c 1.15 06/12/12
@@ -46,10 +46,7 @@
 # include       <i18n.h>
 # include	<schily/sysexits.h>
 
-struct stat Statbuf;
-char SccsError[MAXERRORLEN];
-
-struct sid sid;
+static struct sid sid;
 
 static struct packet gpkt;
 static int	num_files;
@@ -61,10 +58,8 @@ static int	*Cvec;
 static int	Cnt;
 static FILE	*iop;
 
-	void    clean_up __PR((void));
-	void	escdodelt __PR((struct packet *pkt));
-	void	fredck	__PR((struct packet *pkt));
-	void	enter	__PR((struct packet *pkt, int ch, int n, struct sid *sidp));
+static	void    clean_up __PR((void));
+static	void	enter	__PR((struct packet *pkt, int ch, int n, struct sid *sidp));
 
 	int	main __PR((int argc, char **argv));
 static void	comb __PR((char *file));
@@ -88,7 +83,9 @@ register char *argv[];
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
 	
-	/* 
+	sccs_setinsbase(INS_BASE);
+
+	/*
 	 * Set directory to search for general l10n SCCS messages.
 	 */
 #ifdef	PROTOTYPES
@@ -103,7 +100,12 @@ register char *argv[];
 
 	tzset();	/* Set up timezome related vars */
 
+	set_clean_up(clean_up);
 	Fflags = FTLEXIT | FTLMSG | FTLCLN;
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 
 	current_optind = 1;
 	optind = 1;
@@ -124,7 +126,7 @@ register char *argv[];
 				/* this takes care of options given after
 				** file names.
 				*/
-			if((c == EOF)) {
+			if (c == EOF) {
 			   if (optind < argc) {
 				/* if it's due to -- then break; */
 			       if(argv[i][0] == '-' &&
@@ -226,7 +228,6 @@ char *file;
 	int succnt;
 	struct sid *sp;
 	extern char had_dir, had_standinp;
-	extern char *Sflags[];
 	struct stats stats;
 
 	if (setjmp(Fjmp))
@@ -234,6 +235,7 @@ char *file;
 	sinit(&gpkt, file, 1);
 	gpkt.p_verbose = -1;
 	gpkt.p_stdout = stderr;
+	gpkt.p_enter = enter;
 	if (gpkt.p_verbose && (num_files > 1 || had_dir || had_standinp))
 		fprintf(gpkt.p_stdout,"\n%s:\n",gpkt.p_file);
 	if (exists(auxf(gpkt.p_file, 'p')))
@@ -296,7 +298,7 @@ char *file;
 	fprintf(iop,"trap \"rm -f COMB$$ comb$$ s.COMB$$; exit 2\" 1 2 3 15\n");
 	sp = prtget(rdp, Cvec[0], iop, gpkt.p_file);
 	sid_ba(sp,rarg);
-	if ((Val_ptr = Sflags[VALFLAG - 'a']) == NULL)
+	if ((Val_ptr = gpkt.p_sflags[VALFLAG - 'a']) == NULL)
 		Val_ptr = Blank;
 	fprintf(iop, "v=`prs -r%s -d:MR: %s`\n", rarg, gpkt.p_file);
 	fprintf(iop, "if test \"$v\"\n");
@@ -355,7 +357,7 @@ char *file;
 	fprintf(iop, "prs -e %s >>comb$$\n", gpkt.p_file);
 	fprintf(iop, "admin -tcomb$$ s.COMB$$\\\n");
 	for (i = 0; i < NFLAGS; i++)
-		if ((p = Sflags[i]) != NULL)
+		if ((p = gpkt.p_sflags[i]) != NULL)
 		   if ( i != (ENCODEFLAG-'a') )
 			fprintf(iop, " -f%c%s\\\n", i + 'a', p);
 	fprintf(iop, "\n");
@@ -377,7 +379,7 @@ char *file;
 	if (!HADS) {
 		fprintf(iop, "rm -f %s\n", gpkt.p_file);
 		fprintf(iop, "mv s.COMB$$ %s\n", gpkt.p_file);
-		if (!Sflags[VALFLAG - 'a'])
+		if (!gpkt.p_sflags[VALFLAG - 'a'])
 			fprintf(iop, "admin -dv %s\n", gpkt.p_file);
 	} else {
 		fprintf(iop, "set `ls -st s.COMB$$ %s`\n",gpkt.p_file);
@@ -388,7 +390,7 @@ char *file;
 }
 
 /*ARGSUSED*/
-void
+static void
 enter(pkt,ch,n,sidp)
 struct packet *pkt;
 char ch;
@@ -442,22 +444,8 @@ int i;
 	return(i);
 }
 
-void
+static void
 clean_up()
 {
 	ffreeall();
 }
-
-/*ARGSUSED*/
-void
-escdodelt(pkt)			/* dummy for dodelt() */
-	struct packet *pkt;
-{
-}
-
-/*ARGSUSED*/
-void
-fredck(pkt)			/*dummy for dodelt() */
-	struct packet *pkt;
-{
-}
