@@ -2,11 +2,13 @@
  * CDDL HEADER START
  *
  * The contents of this file are subject to the terms of the
- * Common Development and Distribution License (the "License").
- * You may not use this file except in compliance with the License.
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
  *
- * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -25,12 +27,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2014 J. Schilling
  *
- * @(#)delta.c	1.56 11/10/15 J. Schilling
+ * @(#)delta.c	1.61 14/08/09 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)delta.c 1.56 11/10/15 J. Schilling"
+#pragma ident "@(#)delta.c 1.61 14/08/09 J. Schilling"
 #endif
 /*
  * @(#)delta.c 1.40 06/12/12
@@ -42,6 +44,7 @@
 #endif
 
 # define	NEED_PRINTF_J		/* Need defines for js_snprintf()? */
+# include	<schily/resource.h>
 # include	<defines.h>
 # include	<version.h>
 # include	<had.h>
@@ -53,7 +56,6 @@
 # include	<schily/vfork.h>
 # include	<ccstypes.h>
 # include	<schily/sysexits.h>
-# include	<schily/resource.h>
 
 # define	LENMR	60
 
@@ -131,7 +133,9 @@ register char *argv[];
 	 */
 	setlocale(LC_ALL, "");
 	
-	/* 
+	sccs_setinsbase(INS_BASE);
+
+	/*
 	 * Set directory to search for general l10n SCCS messages.
 	 */
 #ifdef	PROTOTYPES
@@ -148,6 +152,10 @@ register char *argv[];
 
 	set_clean_up(clean_up);
 	Fflags = FTLEXIT | FTLMSG | FTLCLN;
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 
 	current_optind = 1;
 	optind = 1;
@@ -332,7 +340,7 @@ char *file;
 	uname(&un);
 	uuname = un.nodename;
 	if (lockit(auxf(gpkt.p_file,'z'),SCCS_LOCK_ATTEMPTS,getpid(),uuname))
-		fatal(gettext("cannot create lock file (cm4)"));
+		efatal(gettext("cannot create lock file (cm4)"));
 
 	sinit(&gpkt,file,1);
 	gpkt.p_enter = enter;
@@ -680,7 +688,7 @@ struct sid *sp, *osp;
 int diffloop;
 int orig_nlines;
 {
-	extern time_t Timenow;
+	extern dtime_t Timenow;
 	struct deltab dt;
 	char str[BUFSIZ];
 	int newser;
@@ -728,7 +736,11 @@ int orig_nlines;
 	else
 		dt.d_pred = opred;
 
+#ifdef	NO_NANOSECS
 	time2dt(&dt.d_dtime, Timenow, 0); /* Timenow was set by dodelt() */
+#else
+	dt.d_dtime = Timenow;		/* Timenow was set by dodelt() */
+#endif
 
         /* Since the NSE always preserves the clear file after delta and
          * makes it read only (no get is required since keywords are not
@@ -883,7 +895,7 @@ struct sid *sp;
 	in = xfopen(auxf(pkt->p_file,'p'), O_RDONLY|O_BINARY);
 	outname = auxf(pkt->p_file, 'q');
 	if ((fd=open(outname, O_WRONLY|O_CREAT|O_EXCL|O_BINARY, 0444)) < 0) {
-	   fatal(gettext("cannot create lock file (cm4)"));
+	   efatal(gettext("cannot create lock file (cm4)"));
 	}
 #ifdef	HAVE_FCHMOD
 	fchmod(fd, (mode_t)0644);
@@ -969,9 +981,12 @@ int difflim;
 
 	xpipe(pfd);
 	if ((i = vfork()) < 0) {
+		int	errsav = errno;
+
 		close(pfd[0]);
 		close(pfd[1]);
-		fatal(gettext("cannot fork, try again (de11)"));
+		errno = errsav;
+		efatal(gettext("cannot fork, try again (de11)"));
 	}
 	else if (i == 0) {
 #ifdef	set_child_standard_fds
@@ -1226,7 +1241,7 @@ struct sid *sidp;
 
 	sid_ba(sidp,str);
 	ap = &pkt->p_apply[n];
-	if (pkt->p_cutoff > pkt->p_idel[n].i_datetime)
+	if (pkt->p_cutoff > pkt->p_idel[n].i_datetime.tv_sec)
 		switch(ap->a_code) {
 	
 		case SX_EMPTY:
