@@ -2,11 +2,13 @@
  * CDDL HEADER START
  *
  * The contents of this file are subject to the terms of the
- * Common Development and Distribution License (the "License").
- * You may not use this file except in compliance with the License.
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
  *
- * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -25,12 +27,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2015 J. Schilling
  *
- * @(#)get.c	1.54 11/10/15 J. Schilling
+ * @(#)get.c	1.60 15/01/25 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)get.c 1.54 11/10/15 J. Schilling"
+#pragma ident "@(#)get.c 1.60 15/01/25 J. Schilling"
 #endif
 /*
  * @(#)get.c 1.59 06/12/12
@@ -46,7 +48,6 @@
 #include	<had.h>
 #include	<i18n.h>
 #include	<schily/utsname.h>
-#include	<schily/utime.h>
 #include	<ccstypes.h>
 #include	<schily/limits.h>
 #include	<schily/sysexits.h>
@@ -73,33 +74,35 @@
  * retrieved).
  */
 
-static struct sid sid;
+static struct packet gpkt;		/* libcomobj data for get()	*/
 
-static struct packet gpkt;
-static unsigned	Ser;
-static int	num_files;
-static int	num_ID_lines;
-static int	cnt_ID_lines;
-static int	expand_IDs;
-static char	*list_expand_IDs;
-static char    *Whatstr = NULL;
-static char	Pfilename[FILESIZE];
-static char	*ilist, *elist, *lfile;
-static time_t	cutoff = MAX_TIME;
-static char	*cutoffstr;
-static char	Gfile[PATH_MAX];
-static char	gfile[PATH_MAX];
-static char	*Type;
-static struct utsname un;
-static char *uuname;
-static char	*Cwd = "";
+static struct sid sid;			/* To hold -r parameter		*/
+static unsigned	Ser;			/* To hold -a parameter		*/
+static char    *Whatstr = NULL;		/* To hold -w parameter		*/
+static char	*ilist;			/* To hold -i parameter		*/
+static char	*elist;			/* To hold -x parameter		*/
+static char	*lfile;			/* To hold -l parameter		*/
+static char	*cutoffstr;		/* To hold -c parameter		*/
+static char	Gfile[PATH_MAX];	/* To hold -G (g. + parameter)	*/
+static char	gfile[PATH_MAX];	/* To hold -G parameter		*/
+static char	*Cwd = "";		/* To hold -C parameter		*/
 
 /* Beginning of modifications made for CMF system. */
 #define CMRLIMIT 128
-static char	cmr[CMRLIMIT];
-static int	cmri = 0;
+static char	cmr[CMRLIMIT];		/* To hold -z parameter		*/
 /* End of insertion */
 
+static int	num_files;		/* arg counter for main()/get()	*/
+static char	Pfilename[FILESIZE];	/* p.filename used in get()	*/
+static time_t	cutoff = MAX_TIME;	/* cutoff time for main()/get()	*/
+
+static int	num_ID_lines;		/* 's' flag for current file	*/
+static int	cnt_ID_lines;		/* current line cmp with above	*/
+static int	expand_IDs;		/* whether to expand ISs	*/
+static char	*list_expand_IDs;	/* 'y' flag for current file	*/
+static char	*Type;			/* 't' flag for current file	*/
+
+
 static	void    clean_up __PR((void));
 static	void	enter	__PR((struct packet *pkt, int ch, int n, struct sid *sidp));
 
@@ -127,12 +130,16 @@ register char *argv[];
 	extern int Fcnt;
 	int current_optind;
 	int no_arg;
+	int	cmri = 0;	/* CMF index for option parsing */
 
 	/*
 	 * Set locale for all categories.
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
-	/* 
+
+	sccs_setinsbase(INS_BASE);
+
+	/*
 	 * Set directory to search for general l10n SCCS messages.
 	 */
 #ifdef	PROTOTYPES
@@ -148,6 +155,10 @@ register char *argv[];
 
 	set_clean_up(clean_up);
 	Fflags = FTLEXIT | FTLMSG | FTLCLN;
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 	current_optind = 1;
 	optind = 1;
 	opterr = 0;
@@ -169,7 +180,7 @@ register char *argv[];
 			}
 			no_arg = 0;
 			i = current_optind;
-		        c = getopt(argc, argv, "-r:c:ebi:x:kl:Lpsmnogta:G:w:zqdC:V(version)");
+		        c = getopt(argc, argv, "-r:c:ebi:x:kl:Lpsmnogta:G:w:zqdC:FV(version)");
 				/* this takes care of options given after
 				** file names.
 				*/
@@ -272,6 +283,7 @@ register char *argv[];
 			case 's':
 			case 't':
 			case 'd':
+			case 'F':
 				if (p) {
 				   sprintf(SccsError,
 				     gettext("value after %c arg (cm8)"), 
@@ -301,7 +313,7 @@ register char *argv[];
 				exit(EX_OK);
 
 			default:
-			   fatal(gettext("Usage: get [-begkmLopst] [-l[p]] [-asequence] [-cdate-time] [-Gg-file] [-isid-list] [-rsid] [-xsid-list] file ..."));
+			   fatal(gettext("Usage: get [-beFgkmLopst] [-l[p]] [-asequence] [-cdate-time] [-Gg-file] [-isid-list] [-rsid] [-xsid-list] file ..."));
 			
 			}
 
@@ -358,6 +370,9 @@ char *file;
 	if (setjmp(Fjmp))
 		return;
 	if (HADE) {
+		struct utsname	un;
+		char		*uuname;
+
 		/*
 		call `sinit' to check if file is an SCCS file
 		but don't open the SCCS file.
@@ -368,7 +383,7 @@ char *file;
 		uname(&un);
 		uuname = un.nodename;
 		if (lockit(auxf(file,'z'),10, getpid(),uuname))
-			fatal(gettext("cannot create lock file (cm4)"));
+			efatal(gettext("cannot create lock file (cm4)"));
 	}
 	/*
 	Open SCCS file and initialize packet
@@ -535,9 +550,16 @@ char *file;
 					xmsg(gfile, NOGETTEXT("get"));
 			}
 		}
-		if ((gpkt.p_flags & (PF_V6 | PF_V6TAGS)) && gpkt.p_hash)
-			if (gpkt.p_hash[ser] != (gpkt.p_ghash & 0xFFFF))
+		if ((gpkt.p_flags & (PF_V6 | PF_V6TAGS)) && gpkt.p_hash) {
+			/*
+			 * SCCS v6 is able to check against SID specific
+			 * checksums, but this will not work in case that
+			 * we use include or exclude lists.
+			 */
+			if (gpkt.p_hash[ser] != (gpkt.p_ghash & 0xFFFF) &&
+			    elist == NULL && ilist == NULL && !HADUCF)
 				fatal(gettext("corrupted file version (co27)"));
+		}
 			
 		if (gpkt.p_gout) {
 			if (fflush(gpkt.p_gout) == EOF)
@@ -582,30 +604,30 @@ char *file;
 		}
 		if (Gfile[0] != '\0' && exists(Gfile) ) {
 			rename(Gfile, gfile);
-#ifdef	HAVE_UTIME
 			if (HADO) {
 				struct tm	tm;
-				struct utimbuf	ut;
+				struct timespec	ts[2];
 				unsigned int	gser;
-				extern time_t	Timenow;
+				extern dtime_t	Timenow;
 
 				gser = sidtoser(&gpkt.p_gotsid, &gpkt);
 
-				ut.actime = Timenow;
-				ut.modtime = gpkt.p_idel[gser].i_datetime;
+				ts[0].tv_sec = Timenow.dt_sec;
+				ts[0].tv_nsec = Timenow.dt_nsec;
+				ts[1].tv_sec = gpkt.p_idel[gser].i_datetime.tv_sec;
+				ts[1].tv_nsec = gpkt.p_idel[gser].i_datetime.tv_nsec;
 				/*
 				 * If we did cheat while scanning the delta
 				 * table and converted the time stamps assuming
 				 * GMT. Fix the resulting error here.
 				 */
 				if (gpkt.p_flags & PF_GMT) {
-					tm = *gmtime(&ut.modtime);
+					tm = *gmtime(&ts[1].tv_sec);
 					tm.tm_isdst = -1;
-					ut.modtime = mktime(&tm);
+					ts[1].tv_sec = mktime(&tm);
 				}
-				utime(gfile, &ut);
+				utimensat(AT_FDCWD, gfile, ts, 0);
 			}
-#endif
 		}
 		setuid(holduid);
 		setgid(holdgid);
@@ -616,6 +638,9 @@ char *file;
 	}
 unlock:
 	if (HADE) {
+		struct utsname	un;
+		char		*uuname;
+
 		copy(auxf(gpkt.p_file,'p'),Pfilename);
 		rename(auxf(gpkt.p_file,'q'),Pfilename);
 		uname(&un);
@@ -805,12 +830,12 @@ static void
 idsetup(pkt)
 register struct packet *pkt;
 {
-	extern time_t Timenow;
+	extern dtime_t Timenow;
 	register int n;
 	register char *p;
 
-	date_ba(&Timenow,Curdate, 0);
-	date_bal(&Timenow,Curdatel, 0);
+	date_ba(&Timenow.dt_sec, Curdate, 0);
+	date_bal(&Timenow.dt_sec, Curdatel, 0);
 	Curdatel[10] = 0;
 	Curtime = &Curdate[9];
 	Curdate[8] = 0;
@@ -820,8 +845,8 @@ register struct packet *pkt;
 		if (pkt->p_apply[n].a_code == APPLY)
 			break;
 	if (n) {
-		date_ba(&pkt->p_idel[n].i_datetime, Chgdate, pkt->p_flags);
-		date_bal(&pkt->p_idel[n].i_datetime, Chgdatel, pkt->p_flags);
+		date_ba(&pkt->p_idel[n].i_datetime.tv_sec, Chgdate, pkt->p_flags);
+		date_bal(&pkt->p_idel[n].i_datetime.tv_sec, Chgdatel, pkt->p_flags);
 	} else {
 #ifdef XPG4
 		FILE	*xf;
@@ -1075,7 +1100,7 @@ char line[];
 						hold_size = new_hold_size;
 						hold = (char *)malloc(hold_size);
 						if (hold == NULL) {
-							fatal(gettext("OUT OF SPACE (ut9)"));
+							efatal(gettext("OUT OF SPACE (ut9)"));
 						}
 					}
 					strcpy(hold,Whatstr);
@@ -1094,13 +1119,13 @@ char line[];
 				copy(pkt->p_file,Dir);
 				dname(Dir);
 				if (getcwd(Olddir,sizeof(Olddir)) == NULL)
-				   fatal(gettext("getcwd() failed (ge20)"));
+				   efatal(gettext("getcwd() failed (ge20)"));
 				if (chdir(Dir) != 0)
-				   fatal(gettext("cannot change directory (ge21)"));
+				   efatal(gettext("cannot change directory (ge21)"));
 			  	if (getcwd(Pname,sizeof(Pname)) == NULL)
-				   fatal(gettext("getcwd() failed (ge20)"));
+				   efatal(gettext("getcwd() failed (ge20)"));
 				if (chdir(Olddir) != 0)
-				   fatal(gettext("cannot change directory (ge21)"));
+				   efatal(gettext("cannot change directory (ge21)"));
 				tp = trans(tp,Pname);
 				tp = trans(tp,"/");
 				tp = trans(tp,(sname(pkt->p_file)));
@@ -1165,7 +1190,7 @@ register char *tp, *str, *rest;
 		tline_size = new_size + strlen(rest) + DEF_LINE_SIZE;
 		tline = (char*) realloc(tline, tline_size);
 		if (tline == NULL) {
-			fatal(gettext("OUT OF SPACE (ut9)"));
+			efatal(gettext("OUT OF SPACE (ut9)"));
 		}
 		tp = tline + filled_size;
 	}
@@ -1228,7 +1253,7 @@ readcopy(name, tp)
 		tline_size = new_size + DEF_LINE_SIZE;
 		tline = (char*) realloc(tline, tline_size);
 		if (tline == NULL) {
-			fatal(gettext("OUT OF SPACE (ut9)"));
+			efatal(gettext("OUT OF SPACE (ut9)"));
 		}
 		tp = tline + filled_size;
 	}
@@ -1276,12 +1301,15 @@ clean_up()
 		unlink(Gfile);
 	}
 	if (HADE) {
-	   uname(&un);
-	   uuname = un.nodename;
-	   if (mylock(auxf(gpkt.p_file,'z'), getpid(), uuname)) {
-	      unlink(auxf(gpkt.p_file,'q'));
-	      unlockit(auxf(gpkt.p_file,'z'), getpid(), uuname);
-	   }
+		struct utsname	un;
+		char		*uuname;
+
+		uname(&un);
+		uuname = un.nodename;
+		if (mylock(auxf(gpkt.p_file,'z'), getpid(), uuname)) {
+			unlink(auxf(gpkt.p_file,'q'));
+			unlockit(auxf(gpkt.p_file,'z'), getpid(), uuname);
+		}
 	}
 	ffreeall();
 }
@@ -1298,14 +1326,14 @@ char *inc, *exc;
 	char *user, *pfile;
 	FILE *in, *out;
 	struct pfile pf;
-	extern time_t Timenow;
+	extern dtime_t Timenow;
 	int fd;
 	char **sflags = pkt->p_sflags;
 
 	if ((user=logname()) == NULL)
 	   fatal(gettext("User ID not in password file (cm9)"));
 	if ((fd=open(auxf(pkt->p_file,'q'),O_WRONLY|O_CREAT|O_EXCL|O_BINARY,0444)) < 0) {
-	   fatal(gettext("cannot create lock file (cm4)"));
+	   efatal(gettext("cannot create lock file (cm4)"));
 	}
 #ifdef	HAVE_FCHMOD
 	fchmod(fd, (mode_t)0644);
@@ -1353,14 +1381,14 @@ char *inc, *exc;
 	 * which is outside the year range specified by POSIX.
 	 */
 #if SIZEOF_TIME_T == 4
-	if (Timenow < Y1969)
+	if (Timenow.dt_sec < Y1969)
 #else
-	if ((Timenow < Y1969) ||
-	    (Timenow >= Y2069))
+	if ((Timenow.dt_sec < Y1969) ||
+	    (Timenow.dt_sec >= Y2069))
 #endif
-		date_bal(&Timenow,line, 0);	/* 4 digit year */
+		date_bal(&Timenow.dt_sec, line, 0);	/* 4 digit year */
 	else
-		date_ba(&Timenow,line, 0);	/* 2 digit year */
+		date_ba(&Timenow.dt_sec, line, 0);	/* 2 digit year */
 	if (fprintf(out,"%s %s %s %s",str1,str2,user,line) == EOF)
 		xmsg(pfile, NOGETTEXT("wrtpfile"));
 	if (inc)
