@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2014 J. Schilling
  *
- * @(#)prs.c	1.39 11/10/21 J. Schilling
+ * @(#)prs.c	1.43 14/08/13 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)prs.c 1.39 11/10/21 J. Schilling"
+#pragma ident "@(#)prs.c 1.43 14/08/13 J. Schilling"
 #endif
 /*
  * @(#)prs.c 1.33 06/12/12
@@ -160,7 +160,9 @@ char *argv[];
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
 
-	/* 
+	sccs_setinsbase(INS_BASE);
+
+	/*
 	 * Set directory to search for general l10n SCCS messages.
 	 */
 #ifdef	PROTOTYPES
@@ -181,6 +183,10 @@ char *argv[];
 	*/
 	set_clean_up(clean_up);
 	Fflags = FTLMSG | FTLCLN | FTLEXIT;
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 
 	/*
 	The following loop processes keyletters and arguments.
@@ -214,7 +220,7 @@ char *argv[];
 				/* this takes care of options given after
 				** file names.
 				*/
-			if((c == EOF)) {
+			if (c == EOF) {
 			   if (optind < argc) {
 				/* if it's due to -- then break; */
 			       if(argv[j][0] == '-' &&
@@ -389,6 +395,14 @@ register	char	*file;
 
 	doflags(&gpkt);
 
+	/*
+	 * If there are SCCS v6 flags or SCCS v6 global meta data,
+	 * we need to skip this here.
+	 */
+	if (gpkt.p_line != NULL &&
+	    gpkt.p_line[0] == CTLCHAR && gpkt.p_line[1] != BUSERTXT)
+		read_to(BUSERTXT,&gpkt);
+
 	/*
 	create auxiliary file for the User Text section
 	*/
@@ -881,13 +895,13 @@ struct	stats	*statp;
 				copy(gpkt.p_file,Dir);
 				dname(Dir);
 				if(getcwd(Olddir,sizeof(Olddir)) == NULL)
-					fatal(gettext("getcwd() failed (prs2)"));
+					efatal(gettext("getcwd() failed (prs2)"));
 				if(chdir(Dir) != 0)
-					fatal(gettext("cannot change directory (prs3)"));
+					efatal(gettext("cannot change directory (prs3)"));
 				if( getcwd(Pname,sizeof(Pname)) == NULL )
-					fatal(gettext("getcwd() failed (prs2)"));
+					efatal(gettext("getcwd() failed (prs2)"));
 				if(chdir(Olddir) != 0)
-					fatal(gettext("cannot change directory (prs3)"));
+					efatal(gettext("cannot change directory (prs3)"));
 				printf("%s/",Pname);
 				printf("%s",sname(gpkt.p_file));
 				break;
@@ -1314,7 +1328,7 @@ struct packet *pkt;
 	print out gotten body :GB:
 	*/
 	if ((i = vfork()) < 0)
-		fatal(gettext("cannot fork, try again"));
+		efatal(gettext("cannot fork, try again"));
 	if (i == 0) {
 		/*
 		perform 'get' and redirect output
@@ -1329,7 +1343,7 @@ struct packet *pkt;
 #ifdef	HAVE_VFORK
 		Fflags |= FTLVFORK;
 #endif
-		fatal(SccsError);
+		efatal(SccsError);
 	}
 	else {
 		wait(&status);
