@@ -2,11 +2,13 @@
  * CDDL HEADER START
  *
  * The contents of this file are subject to the terms of the
- * Common Development and Distribution License (the "License").
- * You may not use this file except in compliance with the License.
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
  *
- * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -25,12 +27,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2014 J. Schilling
  *
- * @(#)rmchg.c	1.36 11/10/15 J. Schilling
+ * @(#)rmchg.c	1.41 14/08/09 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)rmchg.c 1.36 11/10/15 J. Schilling"
+#pragma ident "@(#)rmchg.c 1.41 14/08/09 J. Schilling"
 #endif
 /*
  * @(#)rmchg.c 1.19 06/12/12
@@ -79,9 +81,6 @@
 	non editable cmrs then the change is rejected.
 */
 
-# define COPY 0
-# define NOCOPY 1
-
 static struct sid sid;
 static int num_files;
 static char D_type;
@@ -125,7 +124,9 @@ char *argv[];
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
 	
-	/* 
+	sccs_setinsbase(INS_BASE);
+
+	/*
 	 * Set directory to search for general l10n SCCS messages.
 	 */
 #ifdef	PROTOTYPES
@@ -146,6 +147,10 @@ char *argv[];
 	*/
 	set_clean_up(clean_up);
 	Fflags = FTLMSG | FTLCLN | FTLEXIT;
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 
 	current_optind = 1;
 	optind = 1;
@@ -172,7 +177,7 @@ char *argv[];
 				/* this takes care of options given after
 				** file names.
 				*/
-			if((c == EOF)) {
+			if (c == EOF) {
 			   if (optind < argc) {
 				/* if it's due to -- then break; */
 			       if(argv[i][0] == '-' &&
@@ -362,7 +367,7 @@ char *file;
 	uname(&un);
 	uuname = un.nodename;
 	if (lockit(auxf(file,'z'),SCCS_LOCK_ATTEMPTS,getpid(),uuname))
-		fatal(gettext("cannot create lock file (cm4)"));
+		efatal(gettext("cannot create lock file (cm4)"));
 
 	sinit(&gpkt,file,1);	/* initialize packet and open s-file */
 
@@ -457,7 +462,7 @@ char *file;
 		if (exists(auxf(gpkt.p_file,'p')))
 			rdpfile(&gpkt,&sid);
 
-		flushto(&gpkt,EUSERTXT,COPY);
+		flushto(&gpkt, EUSERTXT, FLUSH_COPY);
 
 		keep = YES;
 		found = NO;
@@ -557,7 +562,7 @@ struct packet *pkt;
 {
 	static int first_time = 1;
 	char	*p;
-	extern time_t Timenow;
+	extern dtime_t Timenow;
 
 	static short int doneesc;
 	/* the CMF -z option is on and MR numb being processed*/
@@ -648,11 +653,11 @@ struct packet *pkt;
 				 * For this reason, we are free to switch to a
 				 * 4-digit year for the initial comment.
 				 */
-				if ((Timenow < Y1969) ||
-				    (Timenow >= Y2038))			/* comment only */
-					date_bal(&Timenow,line, 0);	/* 4 digit year */
+				if ((Timenow.dt_sec < Y1969) ||
+				    (Timenow.dt_sec >= Y2038))			/* comment only */
+					date_bal(&Timenow.dt_sec, line, 0);	/* 4 digit year */
 				else
-					date_ba(&Timenow,line, 0);	/* 2 digit year */
+					date_ba(&Timenow.dt_sec, line, 0);	/* 2 digit year */
 				putline(pkt,line);
 				sprintf(line," %s\n",logname());
 				putline(pkt,line);
@@ -815,7 +820,7 @@ msg(app, name, cmrs, stats, sids, fred, sflags)
 	if( *name != '/' )   /* NOT full path name */
 	{
 		if(getcwd(path,sizeof(path)) == NULL)
-                 	fatal(gettext("getcwd() failed (ge20)"));
+                 	efatal(gettext("getcwd() failed (ge20)"));
 		cat(pname,path,"/",name, (char *)0);
 	}
 	else
@@ -832,7 +837,7 @@ msg(app, name, cmrs, stats, sids, fred, sflags)
 		noexist = 1; /*new termLOG */
 	if(!(fd=fopen(holdfred, NOGETTEXT("ab"))))
 		{
-			fatal(gettext("CASSI msg not writable \n"));
+			efatal(gettext("CASSI msg not writable \n"));
 			return(0);
 		}
 	fprintf(fd,"%s chpost %s q %s sw MID=%s MFS=%s MPA=%s q q\n",app,cmrs,pname,sids,stats,logname());
