@@ -2,13 +2,13 @@
  * CDDL HEADER START
  *
  * The contents of this file are subject to the terms of the
- * Common Development and Distribution License (the "License").
- * You may not use this file except in compliance with the License.
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
  *
- * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
- * See the License for the specific language governing permissions
- * and limitations under the License.
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
@@ -23,12 +23,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2015 J. Schilling
  *
- * @(#)sccs.c	1.55 11/10/01 J. Schilling
+ * @(#)sccs.c	1.68 15/01/25 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)sccs.c 1.55 11/10/01 J. Schilling"
+#pragma ident "@(#)sccs.c 1.68 15/01/25 J. Schilling"
 #endif
 /*
  * @(#)sccs.c 1.85 06/12/12
@@ -61,20 +61,14 @@ extern char *getenv();
 # include	<schily/vfork.h>
 # include	<schily/varargs.h>
 # include	<schily/wait.h>
-#define	error		__js_error__	/* SCCS error differs from schily.h */
 #define	comgetline	__no_comgetl__
 # include	<schily/schily.h>
-#undef	error
 #undef	comgetline
 # include	<schily/pwd.h>
-# include	<schily/utime.h>
 
 static  char **diffs_np, **diffs_ap;
 
-int	Domrs;
-char	*Comments,*Mrs;
-char    Null[1];
-int	didvfork;
+static	int	didvfork;
 
 /*
 **  SCCS.C -- human-oriented front end to the SCCS system.
@@ -375,7 +369,7 @@ static struct sccsprog SccsProg[] =
 	   "get:ixrc -e -b/delta: -s -n -ybranch-place-holder/get:pl -e -t -g" },
 	{ "enter",	ENTER,	NO_SDOT,		NULL },
 	{ "create",	CMACRO,	NO_SDOT,
-	   "enter:abdfmortyz/get:ixbeskcl -t" },
+	   "enter:abdfmortyzV/get:ixbeskcl -t" },
 	{ NULL,		-1,	0,			NULL }
 };
 
@@ -423,7 +417,6 @@ static char path[] = NOGETTEXT("PATH=/usr/xpg4/bin:/usr/ccs/bin:/usr/bin");
 
 static struct	sccsprog	*maincmd = NULL;
 static struct	sccsprog	*curcmd  = NULL;
-struct  stat 		Statbuf;
 
 #ifdef	HAVE_STRSIGNAL
 #else
@@ -473,7 +466,9 @@ main(argc, argv)
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
 	
- 	/* 
+	sccs_setinsbase(INS_BASE);
+
+ 	/*
  	 * Set directory to search for general l10n SCCS messages.
  	 */
 #ifdef	PROTOTYPES
@@ -493,6 +488,10 @@ main(argc, argv)
 #endif
 
 	Fflags = FTLEXIT | FTLMSG | FTLCLN;
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 
 # ifndef SCCSDIR
 	/* Pull "SccsDir" out of the environment variable         */
@@ -662,6 +661,7 @@ main(argc, argv)
 #else
 		fprintf(stderr, gettext("Usage: sccs [ -r ][ -drootprefix ][ -p subdir ]\n\t subcommand [ option ...] [ filename ...]\n"));
 #endif
+		sccshelp(stderr, "basic sub-commands");
 		exit(EX_USAGE);
 		/*NOTREACHED*/
 	}
@@ -674,7 +674,6 @@ static int 		NelemArrSids;
 static int 		size_ap_for_get;
 static int 		cur_num_file;
 static char 	Nsid[50];
-char   		SccsError[MAXERRORLEN];
 static char * 		user_name;
 static char * 		r_option_value;
 static char ** 	ArrSids;
@@ -1577,8 +1576,8 @@ command(argv, forkflag, arg0)
 		{
 			char *cp;
 
-			if (strcmp(*np, "-o") == 0) { /* Keep get -ooption */
-				*np++;
+			if (strcmp(*np, "-o") == 0) { /* Keep get -o option */
+				np++;
 				continue;
 			}	
 			cp = makefile(*argv, SccsDir);
@@ -1731,10 +1730,10 @@ command(argv, forkflag, arg0)
 
 	  case EDITOR: {	/* get -e + call $EDITOR */
 		struct fs {
-			char		*name;
-			struct stat	statb;
-			time_t		mtime;
-			int		nogfile;
+			char		*name;	/* file name to edit	    */
+			struct stat	statb;	/* stat() after "sccs edit" */
+			struct timespec	mtime;	/* mtime before "sccs edit" */
+			int		nogfile; /* miss. before "sccs edit" */
 		};
 		struct fs	_fs[16];
 		struct fs	*fs = _fs;
@@ -1780,18 +1779,23 @@ command(argv, forkflag, arg0)
 				rs = fs;
 			}
 			fs[fsidx].nogfile = 0;
-			fs[fsidx].mtime = (time_t)0;
-			if (!exists(*np))
+			fs[fsidx].mtime.tv_sec = (time_t)0;
+			fs[fsidx].mtime.tv_nsec = 0;
+			if (!exists(*np)) {
 				fs[fsidx].nogfile = 1;
-			else
-				fs[fsidx].mtime = Statbuf.st_mtime;
-
+			} else {
+				fs[fsidx].mtime.tv_sec = Statbuf.st_mtime;
+				fs[fsidx].mtime.tv_nsec = stat_mnsecs(&Statbuf);
+			}
 			xp[0] = *np;
 			xp[1] = NULL;
 			rval = command(xp, FORCE_FORK, "edit");
 			if (rval != 0)			/* Checkout problem */
 				continue;		/* so ignore	    */
 
+			/*
+			 * Save unedited state from after sccs edit *np
+			 */
 			fs[fsidx].name = *np;
 			if (stat(*np, &fs[fsidx].statb) == -1)
 				continue;
@@ -1820,16 +1824,16 @@ command(argv, forkflag, arg0)
 			if (fs[i].nogfile) {
 				rval = command(xp, TRUE, "unget -s");
 			} else {
-#ifdef	HAVE_UTIME
-				struct utimbuf	ut;
-#endif
-				rval = command(xp, TRUE, "unedit");
-#ifdef	HAVE_UTIME
-				ut.actime = statb.st_atime;
-                                ut.modtime = fs[i].mtime;
+				struct timespec	ts[2];
 
-				utime(fs[i].name, &ut);
-#endif
+				rval = command(xp, TRUE, "unedit");
+
+				ts[0].tv_sec = statb.st_atime;
+				ts[0].tv_nsec = stat_ansecs(&statb);
+				ts[1].tv_sec = fs[i].mtime.tv_sec;
+				ts[1].tv_nsec = fs[i].mtime.tv_nsec;
+
+				utimensat(AT_FDCWD, fs[i].name, ts, 0);
 			}
 		}
 		if (rs)
@@ -2017,6 +2021,11 @@ callprog(progpath, flags, argv, forkflag)
 		printf("callprog:\n");
 		for (i = 0; argv[i] != NULL; i++)
 			printf("\t\"%s\"\n", argv[i]);
+		/*
+		 * Avoid flush() problems caused by fork()/vfork()
+		 */
+		if (forkflag)
+			fflush(stdout);
 	}
 # endif
 
@@ -2033,7 +2042,7 @@ callprog(progpath, flags, argv, forkflag)
 		if (Debug)
 			printf("Forking\n");
 # endif
-		i = fork();
+		i = vfork();
 		if (i < 0)
 		{
 			syserr(gettext("cannot fork"));
@@ -2090,7 +2099,7 @@ callprog(progpath, flags, argv, forkflag)
 		didvfork = 1;
 #endif
 	}
-	else if (OutFile >= 0)
+	else if (OutFile >= 0)		/* !forkflag && ... */
 	{
 		syserr(gettext("callprog: setting stdout w/o forking"));
 		if (didvfork)
@@ -3145,9 +3154,9 @@ char *file;
 	printf("\n------- %s -------\n", Rflag ? gfile : tail(gfile));
 	fflush(stdout);
 	if (isfile(pfile)) {
-		getcmd = NOGETTEXT("get:Grcixt -s -k");
+		getcmd = NOGETTEXT("get:Grcixt -o -s -k");
 	} else {
-		getcmd = NOGETTEXT("get:Grcixt -s");
+		getcmd = NOGETTEXT("get:Grcixt -o -s");
 	}
 	strcpy(buf1, template);
 #ifdef	HAVE_MKSTEMP
@@ -3250,13 +3259,6 @@ makegfile(name)
 	return gname;
 }
 
-/* for fatal() */
-void
-clean_up()
-{
-}
-
-
 #ifdef	USE_RECURSIVE
 
 #include <schily/walk.h>
@@ -3354,7 +3356,7 @@ walkfunc(nm, fs, type, state)
 		int f = -1;
 
 		if (!bitset(COLLECT, wp->sccsflags)) {
-			f = open(".", O_RDONLY);
+			f = open(".", O_SEARCH);
 			if (f < 0) {
 				errmsg("Cannot get working directory.\n");
 				state->flags |= WALK_WF_QUIT;
@@ -3645,7 +3647,7 @@ fgetchk(file, dov6, silent)
 		for (p = line;
 		    (p = findbytes(p, idx - (p-line), '\n')) != NULL; p++) {
 			if (pn && p > pn) {
-	err:
+	errout:
 				fclose(inptr);
 				if (silent)
 					return (1);
@@ -3676,7 +3678,7 @@ fgetchk(file, dov6, silent)
 	}
 	if (lastchar != '\n') {
 		if (pn && nline == 0)	/* Found null byte but no newline */
-			goto err;
+			goto errout;
 		if (dov6)
 			return (err);
 		if (!silent)
