@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2014 J. Schilling
  *
- * @(#)unget.c	1.22 11/10/11 J. Schilling
+ * @(#)unget.c	1.25 14/08/09 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)unget.c 1.22 11/10/11 J. Schilling"
+#pragma ident "@(#)unget.c 1.25 14/08/09 J. Schilling"
 #endif
 /*
  * @(#)unget.c 1.24 06/12/12
@@ -101,7 +101,9 @@ char *argv[];
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
 	
-	/* 
+	sccs_setinsbase(INS_BASE);
+
+	/*
 	 * Set directory to search for general l10n SCCS messages.
 	 */
 #ifdef	PROTOTYPES
@@ -118,6 +120,10 @@ char *argv[];
 
 	set_clean_up(clean_up);
 	Fflags = FTLEXIT | FTLMSG | FTLCLN;
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 
 	current_optind = 1;
 	optind = 1;
@@ -145,7 +151,7 @@ char *argv[];
 				/* this takes care of options given after
 				** file names.
 				*/
-			if((c == EOF)) {
+			if (c == EOF) {
 			   if (optind < argc) {
 				/* if it's due to -- then break; */
 			       if(argv[i][0] == '-' &&
@@ -276,7 +282,7 @@ char *file;
 	uname(&un);
 	uuname = un.nodename;
 	if (lockit(auxf(gpkt.p_file,'z'),SCCS_LOCK_ATTEMPTS, getpid(),uuname))
-		fatal(gettext("cannot create lock file (cm4)"));
+		efatal(gettext("cannot create lock file (cm4)"));
 	pp = edpfile(&gpkt,&sid);
 	if (gpkt.p_verbose) {
 		sid_ba(&pp->pf_nsid,str);
