@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2014 J. Schilling
  *
- * @(#)val.c	1.35 11/10/13 J. Schilling
+ * @(#)val.c	1.41 14/08/17 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)val.c 1.35 11/10/13 J. Schilling"
+#pragma ident "@(#)val.c 1.41 14/08/17 J. Schilling"
 #endif
 /*
  * @(#)val.c 1.22 06/12/12
@@ -66,9 +66,6 @@
 # define	FALSE		0
 # define	BLANK(l)	while (!(*l == '\0' || *l == ' ' || *l == '\t')) l++;
 
-#define	COPY	0
-#define	NOCOPY	1
-
 static int	silent;		/* be silent, report only in exit code */
 static int	debug;		/* print debug messages */
 static int	ret_code;	/* prime return code from 'main' program */
@@ -140,7 +137,9 @@ char	*argv[];
 	 */
 	setlocale(LC_ALL, NOGETTEXT(""));
 	
-	/* 
+	sccs_setinsbase(INS_BASE);
+
+	/*
 	 * Set directory to search for general l10n SCCS messages.
 	 */
 #ifdef	PROTOTYPES
@@ -160,7 +159,10 @@ char	*argv[];
 	routine and terminate processing.
 	*/
 	Fflags = FTLMSG | FTLCLN | FTLEXIT;
-
+#ifdef	SCCS_FATALHELP
+	Fflags |= FTLFUNC;
+	Ffunc = sccsfatalhelp;
+#endif
 
 	ret_code = 0;
 	if (argc == 2 && argv[1][0] == '-' && !(argv[1][1])) {
@@ -266,7 +268,7 @@ char	*argv[];
 				/* this takes care of options given after
 				** file names.
 				*/
-			if((c == EOF)) {
+			if (c == EOF) {
 			   if (optind < argc) {
 				/* if it's due to -- then break; */
 			       if(argv[j][0] == '-' &&
@@ -476,7 +478,7 @@ char	*c_name;
 			/*
 			 * Read delta table for get_hashtest()
 			 */
-			if (gpkt.p_flags & PF_V6 && HADUCS)
+			if (gpkt.p_flags & PF_V6 && HADH)
 				get_setup(c_path);
 
 			/*
@@ -531,7 +533,23 @@ char	*c_name;
 							goodn++;
 					}
 				}
-				if (*(--l) != BUSERTXT) {
+
+				/*
+				 * If there are SCCS v6 flags or SCCS v6 global
+				 * meta data, we need to skip this data here.
+				 */
+				if (gpkt.p_line != NULL &&
+				    gpkt.p_line[0] == CTLCHAR &&
+				    gpkt.p_line[1] != BUSERTXT)
+					read_to(BUSERTXT,&gpkt);
+
+				/*
+				 * If we did not find the mandatory begin of
+				 * the user comment area, the file is corrupt.
+				 */
+				if (gpkt.p_line != NULL &&
+				    gpkt.p_line[0] == CTLCHAR &&
+				    gpkt.p_line[1] != BUSERTXT) {
 					fclose(gpkt.p_iop);
 					gpkt.p_iop = NULL;
 					get_close();	/* for SID checksums */
@@ -817,7 +835,7 @@ register struct packet *pkt;
 				pkt->p_line_size += sizeof(buf);
 				pkt->p_line = (char*) realloc(pkt->p_line, pkt->p_line_size);
 				if (pkt->p_line == NULL) {
-					fatal(gettext("OUT OF SPACE (ut9)"));
+					efatal(gettext("OUT OF SPACE (ut9)"));
 				}
 			}
 
@@ -1083,7 +1101,7 @@ register char *d_sid;
 				goods++;
 		}
 		if (gpkt.p_flags & PF_V6 &&
-		    HADUCS && del.type == 'D') {
+		    HADH && del.type == 'D') {
 			int	ser;
 
 			satoi(del.serial, &ser);
@@ -1223,7 +1241,7 @@ get_setup(file)
 
 	if (dodelt(&pk2, &stats, (struct sid *) 0, 0) == 0)
 		fmterr(&pk2);
-	flushto(&pk2, EUSERTXT, NOCOPY);
+	flushto(&pk2, EUSERTXT, FLUSH_NOCOPY);
 	get_off = ftell(pk2.p_iop);
 	slnno = pk2.p_slnno;
 
