@@ -2,11 +2,13 @@
  * CDDL HEADER START
  *
  * The contents of this file are subject to the terms of the
- * Common Development and Distribution License (the "License").
- * You may not use this file except in compliance with the License.
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
  *
- * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -25,14 +27,14 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2014 J. Schilling
  *
- * @(#)defines.h	1.67 11/11/01 J. Schilling
+ * @(#)defines.h	1.76 14/08/09 J. Schilling
  */
 #ifndef	_HDR_DEFINES_H
 #define	_HDR_DEFINES_H
 #if defined(sun)
-#pragma ident "@(#)defines.h 1.67 11/11/01 J. Schilling"
+#pragma ident "@(#)defines.h 1.76 14/08/09 J. Schilling"
 #endif
 /*
  * @(#)defines.h 1.21 06/12/12
@@ -51,6 +53,7 @@
 # include	<schily/fcntl.h>
 # include	<schily/stdio.h>
 # include	<schily/stdlib.h>
+# include	<schily/varargs.h>	/* needed for the vfprintf() prototype */
 # include	<schily/unistd.h>
 # include	<schily/string.h>
 # include	<schily/standard.h>	/* define signed */
@@ -80,17 +83,11 @@ long	timezone;
 #endif
 #define	tzset		xtzset
 
-#if SIZEOF_TIME_T == 4
-
-#define	mktime		xmktime
-#define	localtime	xlocaltime
-#define	gmtime		xgmtime
-
-extern time_t	xmktime		__PR((struct tm *));
-extern struct tm *xlocaltime	__PR((time_t *));
-extern struct tm *xgmtime	__PR((time_t *));
-#endif
-
+/*
+ * Note that the #define above will rename the struct timeb member "timezone"
+ * to "xtimezone" as well as the name of the variable.
+ */
+# include	<schily/timeb.h>
 
 extern int optind, opterr, optopt;
 extern char *optarg;
@@ -114,6 +111,10 @@ extern char *optarg;
 #define	MAXPATHLEN	PATH_MAX
 #endif
 
+#ifdef	NO_SCCS_FATALHELP
+#undef	SCCS_FATALHELP
+#endif
+
 #if	(!defined(HAVE_SNPRINTF) && !defined(NO_SNPRINTF)) || \
 	defined(NEED_PRINTF_J) && !defined(HAVE_PRINTF_J)
 #define	NEED_SCHILY_PRINT	/* We need defines for js_snprintf() */
@@ -128,6 +129,17 @@ extern char *optarg;
 #endif
 #include	<schily/schily.h>
 
+#if SIZEOF_TIME_T == 4
+
+#define	mktime		xmktime
+#define	localtime	xlocaltime
+#define	gmtime		xgmtime
+
+extern time_t	xmktime		__PR((struct tm *));
+extern struct tm *xlocaltime	__PR((time_t *));
+extern struct tm *xgmtime	__PR((time_t *));
+#endif
+
 /*
  * SCCS was written in 1972. It supports 2 digit year strings from 1969..2068.
  * Since the Y2000 fix from 1999, it supports to read 4 digit year strings.
@@ -345,7 +357,7 @@ struct	idel {
 	struct	sid	i_sid;
 	struct	ixg	*i_ixg;
 	int	i_pred;
-	time_t	i_datetime;
+	struct timespec	i_datetime;
 };
 
 # define maxser(pkt)	((pkt)->p_idel->i_pred)
@@ -504,6 +516,7 @@ extern	void	cmrerror __PR((const char *));
 /*
  * Declares for external variables in lib/comobj
  */
+extern	char	*sccs_insbase;
 extern	char	*Comments;
 extern	char	*Mrs;
 extern	int	Domrs;
@@ -590,6 +603,10 @@ extern	void	time2dt	__PR((dtime_t *, time_t, int));
 extern	time_t	gmtoff	__PR((time_t));
 extern	int		ssum __PR((char *, int));
 extern	unsigned int	usum __PR((char *, int));
+extern  void	set_init_path __PR((struct packet *, char *file, char *dir));
+extern	void	sccs_setinsbase __PR((char *));
+extern	int	sccsfatalhelp __PR((char *));
+extern	int	sccshelp __PR((FILE *, char *));
 
 /*
  * Declares for external variables in lib/mpwlib
@@ -641,6 +658,8 @@ extern  char*	get_Sccs_Comments __PR((void));
 extern	int	userexit __PR((int code));
 extern	void	*zrealloc __PR((void *ptr, size_t amt));
 extern	int	urandom	__PR((urand_t *urp));
+extern	void	tv2urand __PR((struct timeval *tvp, urand_t *urp));
+extern	void	urand2tv __PR((urand_t *urp, struct timeval *tvp));
 extern	int	sethome	__PR((void));
 
 #ifdef	DBG_MALLOC
