@@ -1,4 +1,4 @@
-/* @(#)version.h	1.18 11/10/01 Copyright 2007-2011 J. Schilling */
+/* @(#)version.h	1.22 15/01/23 Copyright 2007-2015 J. Schilling */
 
 /*
  * The version for sccs programs
@@ -9,11 +9,11 @@
 #include <schily/mconfig.h>
 
 #ifndef	VERSION
-#define	VERSION	"5.05"
+#define	VERSION	"5.07"
 #endif
 
 #ifndef	VDATE
-#define	VDATE	"2011/10/01"
+#define	VDATE	"2015/01/23"
 #endif
 
 #ifdef	SCHILY_BUILD
@@ -49,5 +49,8 @@
 #ifndef	HOST_OS
 #define	HOST_OS		"unknown"
 #endif
+#ifndef	HOST_SUB
+#define	HOST_SUB	"unknown"
+#endif
 
 #endif	/* _HDR_VERSION_H */
